package tools.descartes.modelextraction.jboss.extraction.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Collections;
import java.util.Map;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.sasl.RealmCallback;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.jboss.as.controller.client.ModelControllerClient;
import org.jboss.dmr.ModelNode;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ProvidingDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemFactory;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ActiveResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.NumberOfParallelProcessingUnits;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ProcessingResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ResourceconfigurationFactory;
import edu.kit.ipd.descartes.mm.resourceconfiguration.SchedulingPolicy;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapeFactory;
import edu.kit.ipd.descartes.mm.resourcetype.ProcessingResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypePackage;

/**
 * Hello world!
 *
 */
public class App {

	private static final String APP_NAME = "testsystem";
	private static final String BASE_PATH = "testmodel";

	public static void main(String[] args) throws IOException {
		new App().run();
	}

	private void run() throws IOException {
		File baseDir = new File("C:\\Users\\spinners\\workspaces\\testmodel\\testmodel");

		String[] servers = new String[] { "10.143.3.175", "10.143.3.176", "10.143.3.238", "10.143.3.240",
				"10.143.3.237", "10.143.3.235" };

		ResourceSet rset = createResourceSet();

		ResourceTypeRepository resTypes = loadResourceTypes(rset);

		DistributedDataCenter dc = createResourceLandscape(resTypes, servers);
		registerWithResourceSet(rset, "testsystem.resourcelandscape", dc);
		saveModel(new File(baseDir, "testsystem.resourcelandscape"), dc);

		String resTypesString = serialize(resTypes);
		try (ModelControllerClient client = createClient(InetAddress.getByName("10.143.3.177"), 9999, "admin", "descartes", "ManagementRealm")) {
			Repository repo = RepositoryFactory.eINSTANCE.createRepository();
			repo.setName(APP_NAME);

			registerWithResourceSet(rset, repo.getName() + ".repository", repo);

			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "mysubsystem");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(serialize(repo));
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);

				ModelNode ret = client.execute(op);

				repo = deserializeRepository(repo.eResource(), ret.get("result").get(0).asString());
			}
			saveModel(new File(baseDir, repo.getName() + ".repository"), repo);

			String repositoryAsString = serialize(repo);
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system = SystemFactory.eINSTANCE.createSystem();
			system.setName(repo.getName());
			registerWithResourceSet(rset, system.getName() + ".system", system);
			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "mysubsystem");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(repositoryAsString);
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);
				ModelNode systemParam = op.get("system");
				systemParam.set(serialize(system));

				ModelNode ret = client.execute(op);

				system = deserializeSystem(system.eResource(), ret.get("result").get(0).asString());
			}

			// Expose all provided interfaces
			for (AssemblyContext ctx : system.getAssemblyContexts()) {
				if (ctx.getEncapsulatedComponent() != null) {
					for (InterfaceProvidingRole componentRole : ctx.getEncapsulatedComponent().getInterfaceProvidingRoles()) {
						InterfaceProvidingRole role = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
						role.setInterface(componentRole.getInterface());
						role.setName(componentRole.getName());
						system.getInterfaceProvidingRoles().add(role);
	
						ProvidingDelegationConnector connector = RepositoryFactory.eINSTANCE
								.createProvidingDelegationConnector();
						connector.setAssemblyContext(ctx);
						connector.setInnerInterfaceProvidingRole(componentRole);
						connector.setOuterInterfaceProvidingRole(role);
						connector.setName(role.getName());
						system.getProvidingDelegationConnectors().add(connector);
					}
				}
			}
			saveModel(new File(baseDir, system.getName() + ".system"), system);

			String systemAsString = serialize(system);
			Deployment deploy = DeploymentFactory.eINSTANCE.createDeployment();
			deploy.setName(APP_NAME);
			deploy.setSystem(system);
			deploy.setTargetResourceLandscape(dc);
			registerWithResourceSet(rset, deploy.getName() + ".deployment", deploy);
			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "mysubsystem");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				op.get("host").set(server);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(repositoryAsString);
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);
				ModelNode systemParam = op.get("system");
				systemParam.set(systemAsString);
				ModelNode resourceLandscapeParam = op.get("resourcelandscape");
				resourceLandscapeParam.set(serialize(dc));
				ModelNode deploymentParam = op.get("deployment");
				deploymentParam.set(serialize(deploy));

				ModelNode ret = client.execute(op);

				deploy = deserializeDeployment(deploy.eResource(), ret.get("result").get(0).asString());
			}
			saveModel(new File(baseDir, deploy.getName() + ".deployment"), deploy);
		}
	}

	private void registerWithResourceSet(ResourceSet rset, String file, EObject content) {
		Resource res = rset.createResource(URI.createPlatformResourceURI(BASE_PATH + "/" + file, false));
		res.getContents().add(content);
	}

	private void saveModel(File file, EObject obj) throws IOException {
		obj.eResource().save(new FileOutputStream(file), Collections.EMPTY_MAP);
	}

	private String serialize(EObject obj) {
		try (ByteArrayOutputStream out = new ByteArrayOutputStream()) {
			obj.eResource().save(out, Collections.EMPTY_MAP);
			return out.toString();
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	private edu.kit.ipd.descartes.mm.applicationlevel.system.System deserializeSystem(Resource resource, String repo) {
		return (edu.kit.ipd.descartes.mm.applicationlevel.system.System) deserialize(resource, repo);
	}

	private Repository deserializeRepository(Resource resource, String repo) {
		return (Repository) deserialize(resource, repo);
	}

	private Deployment deserializeDeployment(Resource resource, String deploy) {
		return (Deployment) deserialize(resource, deploy);
	}

	private DistributedDataCenter createResourceLandscape(ResourceTypeRepository resTypes, String[] servers)
			throws IOException {
		DistributedDataCenter ddc = ResourcelandscapeFactory.eINSTANCE.createDistributedDataCenter();
		ddc.setName("My Company");
		DataCenter dc = ResourcelandscapeFactory.eINSTANCE.createDataCenter();
		dc.setName("My Datacenter");
		dc.setBelongsTo(ddc);

		for (String server : servers) {
			ComputingInfrastructure infra = ResourcelandscapeFactory.eINSTANCE.createComputingInfrastructure();
			infra.setName(server);
			infra.setPartOf(dc);

			NumberOfParallelProcessingUnits cores = ResourceconfigurationFactory.eINSTANCE
					.createNumberOfParallelProcessingUnits();
			cores.setNumber(1);
			cores.setName("Cores");
			ProcessingResourceSpecification spec = ResourceconfigurationFactory.eINSTANCE
					.createProcessingResourceSpecification();
			spec.setNrOfParProcUnits(cores);
			spec.setActiveResourceType(getCPU(resTypes));
			spec.setName("CPU");
			spec.setSchedulingPolicy(SchedulingPolicy.PROCESSOR_SHARING);

			ActiveResourceSpecification cpuRes = ResourceconfigurationFactory.eINSTANCE
					.createActiveResourceSpecification();
			cpuRes.getProcessingResourceSpecifications().add(spec);
			cpuRes.setName(server + "-ActiveResources");
			infra.getConfigSpec().add(cpuRes);
		}

		return ddc;
	}

	private EObject deserialize(Resource resource, String content) {
		try (ByteArrayInputStream out = new ByteArrayInputStream(content.getBytes())) {
			resource.unload();
			resource.load(out, Collections.EMPTY_MAP);
			return resource.getContents().get(0);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	private ResourceSet createResourceSet() {
		ResourcetypePackage.eINSTANCE.eClass();

		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("repository", new XMIResourceFactoryImpl());
		m.put("system", new XMIResourceFactoryImpl());
		m.put("resourcelandscape", new XMIResourceFactoryImpl());
		m.put("resourcetype", new XMIResourceFactoryImpl());
		m.put("deployment", new XMIResourceFactoryImpl());

		ResourceSet resSet = new ResourceSetImpl();
		return resSet;
	}

	private ProcessingResourceType getCPU(ResourceTypeRepository types) {
		for (ResourceType t : types.getResourceTypes()) {
			if (t.getName().equals("CPU")) {
				return (ProcessingResourceType) t;
			}
		}
		return null;
	}

	private ResourceTypeRepository loadResourceTypes(ResourceSet rset) throws IOException {
		Resource resource = rset.createResource(URI
				.createURI("pathmap://DML_APPLICATIONLEVEL_MODELS/defaultresourcetypes.resourcetype"));
		resource.load(getClass().getResourceAsStream("/library/defaultresourcetypes.resourcetype"),
				Collections.EMPTY_MAP);
		return (ResourceTypeRepository) resource.getContents().get(0);
	}

	private static ModelControllerClient createClient(final InetAddress host, final int port, final String username,
			final String password, final String securityRealmName) {

		final CallbackHandler callbackHandler = new CallbackHandler() {

			public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
				for (Callback current : callbacks) {
					if (current instanceof NameCallback) {
						NameCallback ncb = (NameCallback) current;
						ncb.setName(username);
					} else if (current instanceof PasswordCallback) {
						PasswordCallback pcb = (PasswordCallback) current;
						pcb.setPassword(password.toCharArray());
					} else if (current instanceof RealmCallback) {
						RealmCallback rcb = (RealmCallback) current;
						rcb.setText(rcb.getDefaultText());
					} else {
						throw new UnsupportedCallbackException(current);
					}
				}
			}
		};

		return ModelControllerClient.Factory.create(host, port, callbackHandler);
	}
}
