package tools.descartes.prisma.agent.wildfly.callstack.impl;

import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationGroupRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.TraceFactory;
import tools.descartes.prisma.agent.wildfly.callstack.TraceVisitor;

public class DefaultComponentRecord extends ComponentRecord {
	
	private final ConcurrentHashMap<String, InterfaceRecord> interfaces = new ConcurrentHashMap<>();
	
	public DefaultComponentRecord(TraceFactory factory, String component) {
		super(factory, component);
	}
	
	public InterfaceRecord requireInterface(String interf) {
		InterfaceRecord record = interfaces.get(interf);
		if (record == null) {
			record = getTraceFactory().createInterfaceRecord(this, interf);
			if (interfaces.putIfAbsent(interf, record) != null) {
				// a different thread has changed it in the mean time
				record = interfaces.get(interf);
			}
		}
		return record;
	}
	
	public Collection<InterfaceRecord> getInterfaces() {
		return interfaces.values();
	}
	
	@Override
	public boolean isFiltered() {
		return false;
	}
	
	@Override
	public void accept(TraceVisitor visitor, InvocationGroupRecord source) {
//		System.out.println("Visit component " + getComponentName());
		if (source != null) {
			visitor.visitRequiredComponent(source, this);
		} else {
			visitor.visitComponent(this);
		}
		
		for (InterfaceRecord irec : interfaces.values()) {
			irec.accept(visitor, source);
		}
	}
}
