package tools.descartes.prisma.agent.wildfly.extension;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.jboss.as.controller.AttributeDefinition;
import org.jboss.as.controller.OperationDefinition;
import org.jboss.as.controller.PersistentResourceDefinition;
import org.jboss.as.controller.SimpleAttributeDefinition;
import org.jboss.as.controller.SimpleOperationDefinitionBuilder;
import org.jboss.as.controller.SimpleResourceDefinition;
import org.jboss.as.controller.registry.ManagementResourceRegistration;
import org.jboss.dmr.ModelType;

import tools.descartes.prisma.agent.wildfly.filters.FilterDefinition;
import tools.descartes.prisma.agent.wildfly.parameters.ParameterCharacterizationDefinition;

/**
 * @author <a href="mailto:tcerar@redhat.com">Tomaz Cerar</a>
 */
public class SubsystemDefinition extends PersistentResourceDefinition {
    public static final SubsystemDefinition INSTANCE = new SubsystemDefinition();
    
    static final OperationDefinition EXTRACT_PERFORMANCE_MODEL = new SimpleOperationDefinitionBuilder("extract-performance-model", SubsystemExtension.getResourceDescriptionResolver(null))
    		.addParameter(new SimpleAttributeDefinition("application", ModelType.STRING, false))		
    		.addParameter(new SimpleAttributeDefinition("resourcetypes", ModelType.STRING, false))
    		.addParameter(new SimpleAttributeDefinition("repository", ModelType.STRING, false))
    		.addParameter(new SimpleAttributeDefinition("system", ModelType.STRING, false))
    		.addParameter(new SimpleAttributeDefinition("deployment", ModelType.STRING, false))
    		.addParameter(new SimpleAttributeDefinition("resourcelandscape", ModelType.STRING, false))
    		.addParameter(new SimpleAttributeDefinition("host", ModelType.STRING, false))
    		.setReplyType(ModelType.STRING)
    		.build();

    private SubsystemDefinition() {
        super(SubsystemExtension.SUBSYSTEM_PATH,
                SubsystemExtension.getResourceDescriptionResolver(null),
                //We always neeexid to add an 'add' operation
                SubsystemAdd.INSTANCE,
                //Every resource that is added, normally needs a remove operation
                SubsystemRemove.INSTANCE);
    }

    @Override
    public void registerOperations(ManagementResourceRegistration resourceRegistration) {
        super.registerOperations(resourceRegistration);
        //you can register aditional operations here
        resourceRegistration.registerOperationHandler(EXTRACT_PERFORMANCE_MODEL, ModelExtractionHandler.INSTANCE);
    }

    @Override
    public void registerAttributes(ManagementResourceRegistration resourceRegistration) {
        //you can register attributes here
    }

	@Override
	public Collection<AttributeDefinition> getAttributes() {
		return Collections.emptyList();
	}
	
	@Override
	protected List<? extends PersistentResourceDefinition> getChildren() {
		return Arrays.asList(FilterDefinition.INSTANCE, ParameterCharacterizationDefinition.INSTANCE);
	}
}
