package tools.descartes.prisma.agent.wildfly.filters;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.TraceFactory;
import tools.descartes.prisma.agent.wildfly.callstack.impl.DefaultComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.impl.DefaultInterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.impl.DefaultOperationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.impl.FilteredComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.impl.FilteredInterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.impl.FilteredOperationRecord;
import tools.descartes.prisma.agent.wildfly.parameters.ValueCharacterization;

public class FilteredTraceFactory implements TraceFactory {
	
	private final List<ComponentPredicate> componentFilters = new LinkedList<>();
	private final List<InterfacePredicate> interfaceFilters = new LinkedList<>();
	private final List<OperationPredicate> operationFilters = new LinkedList<>();
	
	private Map<String, Map<String, ValueCharacterization>> valueCharacterizations = new HashMap<>();
	
	public void addPredicate(ComponentPredicate filter) {
		System.err.println("Component filter added");
		componentFilters.add(filter);
	}
	
	public void addPredicate(InterfacePredicate filter) {
		System.err.println("Interface filter added");
		interfaceFilters.add(filter);
	}
	
	public void addPredicate(OperationPredicate filter) {
		System.err.println("Operation filter added");
		operationFilters.add(filter);
	}
	
	public void addValueCharacterization(ValueCharacterization charac) {
		System.err.println("Value characterization added");
		Map<String, ValueCharacterization> ops = valueCharacterizations.get(charac.getInterfaceName());
		if (ops == null) {
			ops = new HashMap<>();
			valueCharacterizations.put(charac.getInterfaceName(), ops);
		}
		ops.put(charac.getOperationName(), charac);
	}
	
	public ComponentRecord createComponentRecord(String component) {
		boolean filtered = false;
		for (ComponentPredicate filter : componentFilters) {
			if (filter.apply(component)) {
				filtered = true;
				break;
			}
		}
		if (filtered) {
			return new FilteredComponentRecord(this, component);
		} else {
			return new DefaultComponentRecord(this, component);
		}
	}
	
	public InterfaceRecord createInterfaceRecord(ComponentRecord component, String interf) {
		boolean filtered = false;
		String componentName = component.getComponentName();
		for (InterfacePredicate filter : interfaceFilters) {
			if (filter.apply(componentName, interf)) {
				filtered = true;
				break;
			}
		}
		if (filtered) {
			return new FilteredInterfaceRecord(component, interf);
		} else {
			return new DefaultInterfaceRecord(component, interf);
		}
	}
	
	public OperationRecord createOperationRecord(InterfaceRecord interf, String operation) {
		boolean filtered = false;
		String componentName = interf.getComponent().getComponentName();
		String interfaceName = interf.getInterfaceName();
		for (OperationPredicate filter : operationFilters) {
			if (filter.apply(componentName, interfaceName, operation)) {
				filtered = true;
				break;
			}
		}
		ValueCharacterization valueCharac = null;
		Map<String, ValueCharacterization> charac = valueCharacterizations.get(interf.getInterfaceName());
		if (charac != null) {
			valueCharac = charac.get(operation);
		}
		if (filtered) {
			return new FilteredOperationRecord(interf, operation);
		} else {
			return new DefaultOperationRecord(interf, operation, valueCharac);
		}
	}

}
