package tools.descartes.prisma.agent.wildfly.interceptors;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.sql.SQLException;
import java.util.Deque;
import java.util.Properties;

import org.apache.commons.lang.mutable.MutableLong;
import org.jboss.ejb.client.EJBClientInvocationContext;
import org.jboss.logging.Logger;

import com.mysql.jdbc.Connection;
import com.mysql.jdbc.ResultSetInternalMethods;
import com.mysql.jdbc.Statement;
import com.mysql.jdbc.StatementInterceptorV2;

import tools.descartes.prisma.agent.wildfly.Constants;
import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationGroupRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.ThreadCallStack;

public class JDBCStatementInterceptor implements StatementInterceptorV2 {
	
	private static final Logger log = Logger.getLogger(EJBOutgoingCallInterceptor.class);
	
	private final ThreadMXBean threadBean;
	
	private static final ThreadLocal<MutableLong> invocationTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};
	
	private static final ThreadLocal<MutableLong> startCpuTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};
	
	public JDBCStatementInterceptor() {
		threadBean = ManagementFactory.getThreadMXBean();
		if (threadBean.isCurrentThreadCpuTimeSupported()) {
			threadBean.setThreadCpuTimeEnabled(true);
		}
	}

	@Override
	public void destroy() {
		
	}

	@Override
	public boolean executeTopLevelOnly() {
		return true;
	}

	@Override
	public void init(Connection arg0, Properties arg1) throws SQLException {
	
	}

	@Override
	public ResultSetInternalMethods preProcess(String sql, Statement interceptedStatemenet, Connection con) throws SQLException {	
		Deque<InvocationGroupRecord> threadCallStack = ThreadCallStack.get();
		InvocationGroupRecord currentInvocation = threadCallStack.peek();
		if (currentInvocation != null) {
			String component = "mysql";
			String interf = "sql";
			String operation = "query";
			
			ComponentRecord crec = currentInvocation.requireComponent(component);
			InterfaceRecord irec = crec.requireInterface(interf);
			OperationRecord orec = irec.requireOperation(operation);
			InvocationGroupRecord grec = orec.requireInvocationGroup(0);
			
			threadCallStack.push(grec);
			
//			startCpuTime.get().setValue(threadBean.getCurrentThreadCpuTime());
			invocationTime.get().setValue(System.nanoTime());
		}
		return null;
	}

	@Override
	public ResultSetInternalMethods postProcess(String sql, Statement interceptedStatement, ResultSetInternalMethods res,
		Connection con, int warnings, boolean noIndexUsed, boolean noGoodIndexUsed, SQLException ex) throws SQLException {
	
		long endTime = System.nanoTime();
//		long endCpuTime = threadBean.getCurrentThreadCpuTime();
		Deque<InvocationGroupRecord> threadCallStack = ThreadCallStack.get();
		if (!threadCallStack.isEmpty()) {
			long executionTime = endTime - invocationTime.get().longValue();
//			long cpuTime = endCpuTime - startCpuTime.get().longValue();
			InvocationGroupRecord grec = threadCallStack.pop();
			grec.addInvocation(executionTime, 0, 0);		
				
			if (log.isTraceEnabled()) {
				InvocationGroupRecord caller = threadCallStack.peek();
				log.trace("Outgoing invocation (" + caller.getOperation().getInterface().getInterfaceName() + ", " + caller.getOperation().getOperationName() 
						+ ") -> (" + grec.getOperation().getInterface().getInterfaceName() + ", " + grec.getOperation().getOperationName() + ", " + executionTime + "ms)");				
			}
		}
		return null;
	}

}
