package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import javax.jms.Connection;
import javax.jms.ConnectionConsumer;
import javax.jms.ConnectionMetaData;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.ServerSessionPool;
import javax.jms.Session;
import javax.jms.Topic;

public class ConnectionDelegate implements Connection {

	private final Connection delegate;

	public ConnectionDelegate(Connection delegate) {
		super();
		this.delegate = delegate;
	}

	public Session createSession(boolean transacted, int acknowledgeMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(transacted, acknowledgeMode));
	}

	public Session createSession(int sessionMode) throws JMSException {
		return new SessionDelegate(delegate.createSession(sessionMode));
	}

	public Session createSession() throws JMSException {
		return new SessionDelegate(delegate.createSession());
	}

	public String getClientID() throws JMSException {
		return delegate.getClientID();
	}

	public void setClientID(String clientID) throws JMSException {
		delegate.setClientID(clientID);
	}

	public ConnectionMetaData getMetaData() throws JMSException {
		return delegate.getMetaData();
	}

	public ExceptionListener getExceptionListener() throws JMSException {
		return delegate.getExceptionListener();
	}

	public void setExceptionListener(ExceptionListener listener) throws JMSException {
		delegate.setExceptionListener(listener);
	}

	public void start() throws JMSException {
		delegate.start();
	}

	public void stop() throws JMSException {
		delegate.stop();
	}

	public void close() throws JMSException {
		delegate.close();
	}

	public ConnectionConsumer createConnectionConsumer(Destination destination, String messageSelector,
			ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createConnectionConsumer(destination, messageSelector, sessionPool, maxMessages);
	}

	public ConnectionConsumer createSharedConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createSharedConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}

	public ConnectionConsumer createDurableConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createDurableConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}

	public ConnectionConsumer createSharedDurableConnectionConsumer(Topic topic, String subscriptionName,
			String messageSelector, ServerSessionPool sessionPool, int maxMessages) throws JMSException {
		return delegate.createSharedDurableConnectionConsumer(topic, subscriptionName, messageSelector, sessionPool,
				maxMessages);
	}
}
