package tools.descartes.prisma.agent.wildfly.service;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;
import org.jboss.msc.value.InjectedValue;

import edu.kit.ipd.descartes.mm.applicationlevel.functions.DoubleLiteral;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ExponentialDistribution;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSample;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSampleList;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ProbabilityMassFunction;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.RandomVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ForkAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.HardwareInfrastructure;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationGroupRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;

public class ModelExtractionService implements Service<ModelExtractionService> {
	
	private static final Logger log = Logger.getLogger(ModelExtractionService.class);
	
	private InjectedValue<MonitoringService> monitoring = new InjectedValue<>();
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "modelextraction");
	
	@Override
	public ModelExtractionService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext arg0) throws StartException {
	
	}

	@Override
	public void stop(StopContext arg0) {
		
	}
	
	public void extractApplicationArchitecture(ResourceTypeRepository resourceTypes, Repository repository) {
		ResourceType cpu = null;
		ResourceType delay = null;
		for (ResourceType type : resourceTypes.getResourceTypes()) {
			if (type.getName().equals("CPU")) {
				cpu = type;
			}
			if (type.getName().equals("DELAY")) {
				delay = type;
			}
		}
		Map<String, Interface> interfaces = new HashMap<String, Interface>();
		for (Interface interf : repository.getInterfaces()) {
			interfaces.put(interf.getName(), interf);
		}
		
		for (ComponentRecord crec : monitoring.getValue().getTraceRoot().getComponents()) {
			if (crec.isFiltered()) {
				continue;
			}			
			
			BasicComponent component = getOrCreateComponent(repository, crec);

			for (InterfaceRecord irec : crec.getInterfaces()) {
				if (irec.isFiltered()) {
					continue;
				}
				
				Interface curInterface = getOrCreateInterface(interfaces, irec, repository);
				for (OperationRecord orec : irec.getOperations()) {
					if (orec.isFiltered()) {
						continue;
					}
					for (InvocationGroupRecord grec : orec.getInvocationGroups()) {
						if (grec != null) {
							addSignatureIfAbsent(curInterface, grec);
						}
					}
				}
				
				InterfaceProvidingRole providingRole = addInterfaceProvidingRole(component, irec, curInterface);
				
				addInterfaceRequiringRoles(repository, interfaces, component, irec);		
			
				for (OperationRecord orec : irec.getOperations()) {
					if (orec.isFiltered()) {
						continue;
					}
					for (InvocationGroupRecord grec : orec.getInvocationGroups()) {
						if (grec == null) {
							continue;
						}
						
						FineGrainedBehavior seff;
						Signature curSignature = getSignature(curInterface, grec);
						if (crec.getComponentClass().equals("javax.jms.Queue") || crec.getComponentClass().equals("javax.jms.Topic")) {
							seff = createJmsDestinationImplementation(component, providingRole, curSignature, grec);
						} else {
							seff = createFineGrainedBehavior(component, providingRole, curSignature, grec, cpu, delay);
							
						}
						component.getFineGrainedBehavior().add(seff);
					}
				}
			}			
			repository.getComponents().add(component);
		}
	}

	private InterfaceProvidingRole addInterfaceProvidingRole(BasicComponent component, InterfaceRecord irec,
			Interface curInterface) {
		InterfaceProvidingRole providingRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		providingRole.setInterface(curInterface);
		providingRole.setName(getInterfaceRoleName(component.getName(), irec.getInterfaceName(), false));
		component.getInterfaceProvidingRoles().add(providingRole);
		return providingRole;
	}

	private void addInterfaceRequiringRoles(Repository repository, Map<String, Interface> interfaces,
			BasicComponent component, InterfaceRecord irec) {
		for (OperationRecord orec : irec.getOperations()) {
			if (orec.isFiltered()) {
				continue;
			}
			
			for (InvocationGroupRecord grec : orec.getInvocationGroups()) {
				if (grec == null) {
					continue;
				}
				for (ComponentRecord reqCrec : grec.getRequiredComponents()) {
					if (reqCrec.isFiltered()) {
						continue;
					}

					for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
						if (reqIrec.isFiltered()) {
							continue;
						}								
						Interface curReqInterface = getOrCreateInterface(interfaces, reqIrec, repository);
						
						InterfaceRequiringRole requiringRole = getRequiringRole(component, reqIrec);
						if (requiringRole == null) {							
							requiringRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
							requiringRole.setName(getInterfaceRoleName(component.getName(), reqIrec.getInterfaceName(), true));
							requiringRole.setInterface(curReqInterface);
						}
						
						for (OperationRecord reqOrec : reqIrec.getOperations()) {
							if (reqOrec.isFiltered()) {
								continue;
							}
							
							for (InvocationGroupRecord reqGrec : reqOrec.getInvocationGroups()) {
								if (reqGrec != null) {
									addSignatureIfAbsent(curReqInterface, reqGrec);
								}
							}
						}
						component.getInterfaceRequiringRoles().add(requiringRole);
					}
				}
			}
		}
	}
	
	private FineGrainedBehavior createJmsDestinationImplementation(RepositoryComponent component, InterfaceProvidingRole providingRole, Signature signature, InvocationGroupRecord grec) {
		FineGrainedBehavior seff = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
		seff.setInterfaceProvidingRole(providingRole);
		seff.setDescribedSignature(signature);
		
		ComponentInternalBehavior behavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
		seff.setBehavior(behavior);
		
		ForkAction fork = ServicebehaviorFactory.eINSTANCE.createForkAction();
		fork.setWithSynchronizationBarrier(false);
		
		ComponentInternalBehavior forkBehavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
		fork.getForkedBehaviors().add(forkBehavior);
		
		for (ComponentRecord reqCrec : grec.getRequiredComponents()) {
			if (reqCrec.isFiltered()) {
				continue;
			}
			for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
				if (reqIrec.isFiltered()) {
					continue;
				}

				for (OperationRecord reqOrec : reqIrec.getOperations()) {
					if (reqOrec.isFiltered()) {
						continue;
					}
					for (InvocationGroupRecord reqGrec : reqOrec.getInvocationGroups()) {
						if (reqGrec != null && reqGrec.getInvocationCount() > 0) {
							ExternalCall extOperation = ServicebehaviorFactory.eINSTANCE.createExternalCall();
							InterfaceRequiringRole extInterfaceRole = getRequiringRole(component, reqIrec);
							extOperation.setName(getExternalCallName(reqIrec, reqOrec));
							extOperation.setInterfaceRequiringRole(extInterfaceRole);
							extOperation.setSignature(getSignature(extInterfaceRole.getInterface(), reqGrec));
							
							ExternalCallAction callAction = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
							callAction.setExternalCall(extOperation);
							forkBehavior.getActions().add(callAction);
						}
					}
				}
			}
		}
		
		return seff;
	}
	
	private FineGrainedBehavior createFineGrainedBehavior(RepositoryComponent component, InterfaceProvidingRole providingRole, Signature signature, InvocationGroupRecord grec, ResourceType cpu, ResourceType delay) {
		FineGrainedBehavior seff = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
		seff.setInterfaceProvidingRole(providingRole);
		seff.setDescribedSignature(signature);
		
		ComponentInternalBehavior behavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
		seff.setBehavior(behavior);
		
		InternalAction internalProcessing = ServicebehaviorFactory.eINSTANCE.createInternalAction();					
		ResourceDemand cpuDemand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
		cpuDemand.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
		cpuDemand.setResourceType(cpu);
		internalProcessing.getResourceDemand().add(cpuDemand);
		behavior.getActions().add(internalProcessing);
		
		for (ComponentRecord reqCrec : grec.getRequiredComponents()) {
			if (reqCrec.isFiltered()) {
				continue;
			}
			for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
				if (reqIrec.isFiltered()) {
					continue;
				}

				for (OperationRecord reqOrec : reqIrec.getOperations()) {
					if (reqOrec.isFiltered()) {
						continue;
					}
					for (InvocationGroupRecord reqGrec : reqOrec.getInvocationGroups()) {
						if (reqGrec != null && reqGrec.getInvocationCount() > 0) {		
							LoopAction loop = ServicebehaviorFactory.eINSTANCE.createLoopAction();
							behavior.getActions().add(loop);
							
							LoopIterationCount invocations = ServicebehaviorFactory.eINSTANCE.createLoopIterationCount();
							invocations.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
							RandomVariable freqRV = FunctionsFactory.eINSTANCE.createRandomVariable();
							ProbabilityMassFunction pmf = FunctionsFactory.eINSTANCE.createProbabilityMassFunction();
							IntSampleList samples = FunctionsFactory.eINSTANCE.createIntSampleList();

							// Determine invocation count: currently it does not consider the distribution
							double avgCalls = ((double)reqGrec.getInvocationCount()) / grec.getInvocationCount();
							if (Math.ceil(avgCalls) == Math.floor(avgCalls)) {
								IntSample count = FunctionsFactory.eINSTANCE.createIntSample();
								count.setProbability(BigDecimal.valueOf(1.0));
								count.setValue(BigInteger.valueOf((long)avgCalls));
								samples.getItems().add(count);
							} else {									
								IntSample count = FunctionsFactory.eINSTANCE.createIntSample();
								count.setProbability(BigDecimal.valueOf(Math.ceil(avgCalls) - avgCalls));
								count.setValue(BigInteger.valueOf((long)Math.floor(avgCalls)));
								samples.getItems().add(count);
								count = FunctionsFactory.eINSTANCE.createIntSample();
								count.setProbability(BigDecimal.valueOf(1 - (Math.ceil(avgCalls) - avgCalls)));
								count.setValue(BigInteger.valueOf((long)Math.ceil(avgCalls)));
								samples.getItems().add(count);
							}
							
							pmf.setSamples(samples);
							freqRV.setProbFunction(pmf);
							invocations.setExplicitDescription(freqRV);
							loop.setLoopIterationCount(invocations);
							
							ComponentInternalBehavior loopActions = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
							loop.setLoopBodyBehavior(loopActions);
							
							InternalAction networkDelay = ServicebehaviorFactory.eINSTANCE.createInternalAction();
							ResourceDemand network = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
							network.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
							network.setResourceType(delay);
							RandomVariable var = FunctionsFactory.eINSTANCE.createRandomVariable();
							DoubleLiteral value = FunctionsFactory.eINSTANCE.createDoubleLiteral();
							value.setValue(BigDecimal.valueOf(0));
							var.setProbFunction(value);											
							network.setExplicitDescription(var);
							networkDelay.getResourceDemand().add(network);
							
							loopActions.getActions().add(networkDelay);
							
							ExternalCall extOperation = ServicebehaviorFactory.eINSTANCE.createExternalCall();
							InterfaceRequiringRole extInterfaceRole = getRequiringRole(component, reqIrec);
							extOperation.setName(getExternalCallName(reqIrec, reqOrec));
							extOperation.setInterfaceRequiringRole(extInterfaceRole);
							extOperation.setSignature(getSignature(extInterfaceRole.getInterface(), reqGrec));
							
							ExternalCallAction callAction = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
							callAction.setExternalCall(extOperation);
							
							loopActions.getActions().add(callAction);									
						}
					}
				}
			}
		}
		return seff;
	}
	
	public void extractSystemAssembly(Repository repository,
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system) {
		for (ComponentRecord crec : monitoring.getValue().getTraceRoot().getComponents()) {
			if (crec.isFiltered()) {
				continue;
			}
			
			AssemblyContext context = getOrCreateAssemblyContext(system, crec);
			RepositoryComponent comp = context.getEncapsulatedComponent();
			if (comp == null) {
				comp = getComponent(repository, crec);				
				context.setEncapsulatedComponent(comp);
			}
			for (InterfaceRecord irec : crec.getInterfaces()) {
				if (irec.isFiltered()) {
					continue;
				}
				for (OperationRecord orec : irec.getOperations()) {
					if (orec.isFiltered()) {
						continue;
					}
					for (InvocationGroupRecord grec : orec.getInvocationGroups()) {
						if (grec == null) {
							continue;
						}
						for (ComponentRecord reqCrec : grec.getRequiredComponents()) {
							if (reqCrec.isFiltered()) {
								continue;
							}
							for (InterfaceRecord reqIrec : reqCrec.getInterfaces()) {
								if (reqIrec.isFiltered()) {
									continue;
								}
								InterfaceRequiringRole reqRole = getRequiringRole(comp, reqIrec);
								AssemblyContext calledContext = getOrCreateAssemblyContext(system, reqCrec);						
								
								AssemblyConnector connector = getOrCreateAssemblyConnector(system, context, calledContext, reqRole);
								connector.setName(getAssemblyConnectorName(reqCrec));
								if (connector.getInterfaceProvidingRole() == null && calledContext.getEncapsulatedComponent() != null) {
									InterfaceProvidingRole role = getProvidingRole(calledContext.getEncapsulatedComponent(), reqIrec.getInterfaceName());
									connector.setInterfaceProvidingRole(role);
								}
								system.getAssemblyConnectors().add(connector);
							}
						}
					}
				}
			}
			for (AssemblyConnector connector : system.getAssemblyConnectors()) {
				if (connector.getProvidingAssemblyContext().equals(context)) {
					if (connector.getInterfaceProvidingRole() == null) {
						String interfaceName = connector.getInterfaceRequiringRole().getInterface().getName();
						connector.setInterfaceProvidingRole(getProvidingRole(comp, interfaceName));
					}
				}
			}
		}
		
	}
	
	public void extractDeployment(Repository repository,
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system, Deployment deployment, String hostName) {
		DistributedDataCenter ddc = deployment.getTargetResourceLandscape();
		ComputingInfrastructure host = null;
		for (DataCenter dc : ddc.getConsistsOf()) {
			for (HardwareInfrastructure hw : dc.getContains()) {
				System.out.println(hw.getName());
				if (hw instanceof ComputingInfrastructure) {
					if (hw.getName().equals(hostName)) {
						host = (ComputingInfrastructure)hw;
						break;
					}
				}
			}
		}
		if (host == null) {
			log.error("Computing infrastructure with name: " + host +" not found.");
		}
		
		for (ComponentRecord crec : monitoring.getValue().getTraceRoot().getComponents()) {
			if (crec.isFiltered()) {
				continue;
			}
			DeploymentContext ctx = DeploymentFactory.eINSTANCE.createDeploymentContext();
			ctx.setName(crec.getComponentName() + "->" + host.getName());
			ctx.setResourceContainer(host);
			ctx.setAssemblyContext(getAssemblyContext(system, crec));
			deployment.getDeploymentContexts().add(ctx);
		}
	}
	
	public InjectedValue<MonitoringService> getMonitoringService() {
		return monitoring;
	}
	
	private AssemblyContext getAssemblyContext(edu.kit.ipd.descartes.mm.applicationlevel.system.System system, ComponentRecord record) {
		for (AssemblyContext ctx : system.getAssemblyContexts()) {
			if (ctx.getName().equals(record.getComponentName())) {
				return ctx;
			}
		}
		return null;
	}
	
	private AssemblyConnector getOrCreateAssemblyConnector(edu.kit.ipd.descartes.mm.applicationlevel.system.System system, AssemblyContext reqCtx, AssemblyContext provCtx, InterfaceRequiringRole reqRole) {
		for (AssemblyConnector con : system.getAssemblyConnectors()) {
			if (con.getRequiringAssemblyContext().equals(reqCtx)) {
				if (con.getProvidingAssemblyContext().equals(provCtx)) {
					if (con.getInterfaceRequiringRole().equals(reqRole)) {
						return con;
					}
				}
			}
		}
		
		AssemblyConnector connector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
		connector.setRequiringAssemblyContext(reqCtx);
		connector.setProvidingAssemblyContext(provCtx);
		connector.setInterfaceRequiringRole(reqRole);
		return connector;
	}
	
	private AssemblyContext getOrCreateAssemblyContext(edu.kit.ipd.descartes.mm.applicationlevel.system.System system, ComponentRecord record) {
		AssemblyContext existing = getAssemblyContext(system, record);
		if (existing == null) {
			AssemblyContext newAssembly = RepositoryFactory.eINSTANCE.createAssemblyContext();
			newAssembly.setName(record.getComponentName());
			system.getAssemblyContexts().add(newAssembly);
			return newAssembly;
		}
		return existing;
	}
	
	private BasicComponent getOrCreateComponent(Repository repository, ComponentRecord crec) {
		RepositoryComponent component = getComponent(repository, crec);
		if (component == null) {
			component = RepositoryFactory.eINSTANCE.createBasicComponent();
			component.setName(getComponentName(crec));
		}
		return (BasicComponent)component;
	}
	
	private RepositoryComponent getComponent(Repository repository, ComponentRecord crec) {
		for (RepositoryComponent comp : repository.getComponents()) {
			if (comp.getName().equals(crec.getComponentClass())) {
				return comp;
			}
		}
		return null;
	}
	
	private InterfaceProvidingRole getProvidingRole(RepositoryComponent component, String interf) {
		String roleName = getInterfaceRoleName(component.getName(), interf, false);
		for (InterfaceProvidingRole role : component.getInterfaceProvidingRoles()) {
			if (role.getName().equals(roleName)) {
				return role;
			}
		}
		return null;
	}
	
	private InterfaceRequiringRole getRequiringRole(RepositoryComponent component, InterfaceRecord irec) {
		for (InterfaceRequiringRole role : component.getInterfaceRequiringRoles()) {
			if (role.getName().equals(getInterfaceRoleName(component.getName(), irec.getInterfaceName(), true))) {
				return role;
			}
		}
		return null;
	}
	
	private Interface getOrCreateInterface(Map<String, Interface> existingInterfaces, InterfaceRecord crec, Repository repository) {
		Interface curInterface = existingInterfaces.get(getInterfaceName(crec));
		if (curInterface == null) {
			curInterface = RepositoryFactory.eINSTANCE.createInterface();
			curInterface.setName(getInterfaceName(crec));
			repository.getInterfaces().add(curInterface);
			existingInterfaces.put(curInterface.getName(), curInterface);
		}
		return curInterface;
	}
	
	private Signature getSignature(Interface intf, InvocationGroupRecord record) {
		String signatureName = getSignatureName(record);
		for (Signature sig : intf.getSignatures()) {
			if (sig.getName().equals(signatureName)) {
				return sig;
			}
		}
		return null;
	}
	
	private void addSignatureIfAbsent(Interface intf, InvocationGroupRecord record) {		
		Signature operationSignature = getSignature(intf, record);
		if (operationSignature == null) {
			operationSignature = RepositoryFactory.eINSTANCE.createSignature();
			operationSignature.setName(getSignatureName(record));
			intf.getSignatures().add(operationSignature);
		}
	}
	
	private String getSignatureName(InvocationGroupRecord record) {
		String name = record.getOperation().getOperationName();
		if (!record.getName().isEmpty()) {
			name += "_" + record.getName();
		}
		return name;
	}
	
	private String getExternalCallName(InterfaceRecord crec, OperationRecord orec) {
		return crec.getInterfaceName() + "#" + orec.getOperationName();
	}
	
	private String getInterfaceName(InterfaceRecord crec) {
		return crec.getInterfaceName();
	}
	
	private String getInterfaceRoleName(String component, String interf, boolean req) {
		return component + "#" + (req ? "Req:" : "Prov:") + interf;
	}

	private String getComponentName(ComponentRecord crec) {
		return crec.getComponentClass();
	}
	
	private String getAssemblyConnectorName(ComponentRecord crec) {
		return crec.getComponentName() + "Connector";
	}
	
	private String getId(String name) {
		return name.replace("/", "_").replace(":", "_");
	}
}
