package tools.descartes.prisma.core.commands;

import java.util.Collection;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;

public class AddAssemblyConnectorCommand extends ModelCommand<AssemblyConnector> {
	
	private final AddAssemblyContextCommand addSourceContext;
	private final AddInterfaceRequiringRoleCommand addRequiringRole;
	private final AddAssemblyContextCommand addTargetContext;
	private final AddInterfaceProvidingRoleCommand addProvidingRole;
	
	public AddAssemblyConnectorCommand(AddAssemblyContextCommand sourceContext, AddInterfaceRequiringRoleCommand requiringRole,
			AddAssemblyContextCommand targetContext, AddInterfaceProvidingRoleCommand providingRole) {
		this.addSourceContext = sourceContext;
		this.addRequiringRole = requiringRole;
		this.addTargetContext = targetContext;
		this.addProvidingRole = providingRole;
	}

	@Override
	public AssemblyConnector apply(CommandContext ctx) {
		AssemblyContext sourceContext = addSourceContext.apply(ctx);
		InterfaceRequiringRole requiringRole = addRequiringRole.apply(ctx);
		AssemblyContext targetContext = addTargetContext.apply(ctx);
		InterfaceProvidingRole providingRole = addProvidingRole.apply(ctx);
		
		System system = ctx.getSystem();
		
		AssemblyConnector connector = find(system.getAssemblyConnectors(), sourceContext, requiringRole, targetContext, providingRole);
		if (connector == null) {
			connector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
			connector.setName(sourceContext.getName() + " -> " + targetContext.getName());
			connector.setRequiringAssemblyContext(sourceContext);
			connector.setProvidingAssemblyContext(targetContext);
			connector.setInterfaceRequiringRole(requiringRole);
			connector.setInterfaceProvidingRole(providingRole);
			system.getAssemblyConnectors().add(connector);
		}
		return connector;
	}
	
	private AssemblyConnector find(Collection<AssemblyConnector> connectors, AssemblyContext sourceContext, 
			InterfaceRequiringRole requiringRole, AssemblyContext targetContext, InterfaceProvidingRole providingRole) {
		for (AssemblyConnector con : connectors) {
			if (con.getRequiringAssemblyContext().equals(sourceContext)) {
				if (con.getInterfaceRequiringRole().equals(requiringRole)) {
					if (con.getProvidingAssemblyContext().equals(targetContext)) {
						if (con.getInterfaceProvidingRole().equals(providingRole)) {
							return con;
						}
					}
				}
			}
		}
		return null;
	}

}
