package tools.descartes.prisma.core.commands;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;

public class AddAssemblyContextCommand extends ModelCommand<AssemblyContext> {
	
	private final String assemblyContextName;
	private final AddComponentCommand component;
	
	public AddAssemblyContextCommand(String assemblyContextName, AddComponentCommand component) {
		this.assemblyContextName = assemblyContextName;
		this.component = component;
	}	

	@Override
	public AssemblyContext apply(CommandContext ctx) {
		RepositoryComponent encapsulated = component.apply(ctx);		
		
		System system = ctx.getSystem();
		AssemblyContext assembly = find(system.getAssemblyContexts(), assemblyContextName);		

		if (assembly == null) {
			assembly = RepositoryFactory.eINSTANCE.createAssemblyContext();
			assembly.setId("AssemblyContext_" + getId(assemblyContextName));
			assembly.setName(assemblyContextName);
			system.getAssemblyContexts().add(assembly);
		}

		RepositoryComponent current = assembly.getEncapsulatedComponent();
		if (current != encapsulated) {
			if (current != null) {
				// TODO logging
				//log.warn("Encapsulated component of assembly context " + assemblyContextName + " will be overwritten (current=" + current.getName() + ", new=" + encapsulated.getName() + ").");
			}
			assembly.setEncapsulatedComponent(encapsulated);
		}			
		return assembly;
	}
}
