package tools.descartes.prisma.core.commands;

import java.util.Collection;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;

public class AddFineGrainedBehaviorCommand extends ModelCommand<FineGrainedBehavior> {
	
	private final AddBasicComponentCommand addComponent;
	private final AddInterfaceProvidingRoleCommand addProvidingRole;
	private final AddSignatureCommand addSignature;

	public AddFineGrainedBehaviorCommand(AddBasicComponentCommand addComponent, AddInterfaceProvidingRoleCommand addProvidingRole, AddSignatureCommand addSignature) {
		this.addComponent = addComponent;
		this.addProvidingRole = addProvidingRole;
		this.addSignature = addSignature;
	}

	@Override
	public FineGrainedBehavior apply(CommandContext ctx) {
		BasicComponent component = addComponent.apply(ctx);
		InterfaceProvidingRole providingRole = addProvidingRole.apply(ctx);
		Signature signature = addSignature.apply(ctx);
		
		FineGrainedBehavior behavior = find(component.getFineGrainedBehavior(), providingRole, signature);
		if (behavior == null) {
			behavior = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
			behavior.setInterfaceProvidingRole(providingRole);
			behavior.setDescribedSignature(signature);
			behavior.setBehavior(ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior());
		}
		return behavior;
	}
	
	private FineGrainedBehavior find(Collection<FineGrainedBehavior> behaviors, InterfaceProvidingRole providingRole, Signature signature) {
		for (FineGrainedBehavior cur : behaviors) {
			if (cur.getInterfaceProvidingRole().equals(providingRole)) {
				if (cur.getDescribedSignature().equals(signature)) {
					return cur;
				}
			}
		}
		return null;
	}

}
