package tools.descartes.prisma.core.commands;

import org.eclipse.emf.cdo.eresource.CDOResource;
import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.ecore.EObject;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.usageprofile.UsageProfile;
import tools.descartes.prisma.core.ApplicationModel;

public class CommandContext {
	
	private final ApplicationModel application;
	private final CDOTransaction transaction;
	
	private DistributedDataCenter resourcelandscape = null;
	private Repository repository = null;
	private System system = null;
	private Deployment deployment = null;
	private UsageProfile usageProfile = null;
	
	public CommandContext(ApplicationModel application, CDOTransaction transaction) {
		this.application = application;
		this.transaction = transaction;		
	}

	public DistributedDataCenter getResourceLandscape() {
		if (resourcelandscape == null) {
			resourcelandscape = (DistributedDataCenter)load(application.getResourceLandscapePath());
		}
		return resourcelandscape;
	}
	
	public Repository getRepository() {
		if (repository == null) {
			repository = (Repository)load(application.getRepositoryPath());
		}
		return repository;
	}
	
	public System getSystem() {
		if (system == null) {
			system = (System)load(application.getSystemPath());
		}
		return system;
	}
	
	public Deployment getDeployment() {
		if (deployment == null) {
			deployment = (Deployment)load(application.getDeploymentPath());
		}
		return deployment;
	}
	
	public UsageProfile getUsageProfile() {
		if (usageProfile == null) {
			usageProfile = (UsageProfile)load(application.getUsageProfilePath());
		}
		return usageProfile;
	}

	private EObject load(String path) {
		CDOResource resource = transaction.getResource(path);
		return resource.getContents().get(0);
	}
}
